const Ci = Components.interfaces;

const CLASS_ID = Components.ID("B9149514-DBC9-11DC-B9BD-1AAB56D89593");
const CLASS_NAME = "Mahalo Share Search Suggest";
const CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=mahalo-share-search-suggest";

// Implements nsIAutoCompleteResult
function SimpleAutoCompleteResult(searchString, searchResult,
                                  defaultIndex, errorDescription,
                                  results, comments) {
  this._searchString = searchString;
  this._searchResult = searchResult;
  this._defaultIndex = defaultIndex;
  this._errorDescription = errorDescription;
  this._results = results;
  this._comments = comments;
}

SimpleAutoCompleteResult.prototype = {
  _searchString: "",
  _searchResult: 0,
  _defaultIndex: 0,
  _errorDescription: "",
  _results: [],
  _comments: [],
  _request : null,
  stubs : [],

  /**
   * The original search string
   */
  get searchString() {
    return this._searchString;
  },

  /**
   * The result code of this result object, either:
   *         RESULT_IGNORED   (invalid searchString)
   *         RESULT_FAILURE   (failure)
   *         RESULT_NOMATCH   (no matches found)
   *         RESULT_SUCCESS   (matches found)
   */
  get searchResult() {
    return this._searchResult;
  },

  /**
   * Index of the default item that should be entered if none is selected
   */
  get defaultIndex() {
    return this._defaultIndex;
  },

  /**
   * A string describing the cause of a search failure
   */
  get errorDescription() {
    return this._errorDescription;
  },

  /**
   * The number of matches
   */
  get matchCount() {
    return this._results.length;
  },

  /**
   * Get the value of the result at the given index
   */
  getValueAt: function(index) {
    return this._results[index];
  },

  /**
   * Get the comment of the result at the given index
   */
  getCommentAt: function(index) {
    return this._comments[index];
  },

  /**
   * Get the style hint for the result at the given index
   */
  getStyleAt: function(index) {
    if (!this._comments[index])
      return null;  // not a category label, so no special styling

    if (index == 0)
      return "suggestfirst";  // category label on first line of results

    return "suggesthint";   // category label on any other line of results
  },

  /**
   * Remove the value at the given index from the autocomplete results.
   * If removeFromDb is set to true, the value should be removed from
   * persistent storage as well.
   */
  removeValueAt: function(index, removeFromDb) {
    this._results.splice(index, 1);
    this._comments.splice(index, 1);
  },

  QueryInterface: function(aIID) {
    if (!aIID.equals(Ci.nsIAutoCompleteResult) && !aIID.equals(Ci.nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


// Implements nsIAutoCompleteSearch
function SimpleAutoCompleteSearch() {
}

SimpleAutoCompleteSearch.prototype = {
  /*
   * Search for a given string and notify a listener (either synchronously
   * or asynchronously) of the result
   *
   * @param searchString - The string to search for
   * @param searchParam - An extra parameter
   * @param previousResult - A previous result to use for faster searchinig
   * @param listener - A listener to notify when the search is complete
   */
  startSearch: function(searchString, searchParam, result, listener) {
	var obj = this;
	
	if (!this._request) {
	  this._request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
	  this._request.QueryInterface(Components.interfaces.nsIDOMEventTarget);
	  this._request.QueryInterface(Components.interfaces.nsIXMLHttpRequest);
	  this._request.parent = this;
	}	
	
	if (!this.stubs) {
		this.stubs = [];
	}
	
	searchString = searchString.replace(/^s+|\s+$/g, "").toLowerCase();
	
	if (searchString.length >= 2) {
		// Get the first two letters.
		var prefix = searchString.substr(0, 2);
		
		if (obj.stubs[prefix]) {
			// If we already have that cached, search that array for matches
			obj.processResults(searchString, obj.stubs[prefix], listener);
		}
		else {
			// Otherwise request it, and then search.

			// Request the stubs file for this prefix.
			var charCode1 = prefix.charCodeAt(0);
			if (charCode1.toString().length < 3) charCode1 = "0" + charCode1;
			
			var charCode2 = prefix.charCodeAt(1);
			if (charCode2.toString().length < 3) charCode2 = "0" + charCode2;
			
			var charCodes = charCode1 + "" + charCode2;
			
			var url = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.mahalo.share.").getCharPref("uris.autocompleteService").replace("%s", charCodes);
			
			this._request.open("GET", url, true);
			this._request.parent = this;
			
			var theObject = this;
			
			this._request.onreadystatechange = function (event) {
				if (obj._request.readyState == 4){
					try {
						if (obj._request.status == 200){
							try {
								eval("obj." + obj._request.responseText + ";");
							} catch (e) {
								// logMhoShareMsg1("Malformed response from auto-complete: " + obj._request.responseText);
							}
						}
						else {
							// logMhoShareMsg1("Response: " + obj._request.status + ": " + obj._request.responseText);
						}
					} catch (e) { // Error getting the status 
						// logMhoShareMsg1("Error: " + e);
					}
					
					obj.processResults(searchString, obj.stubs[prefix], listener);
				}
			};

			this._request.send(null);
		}
	}
  },

  /*
   * Stop an asynchronous search that is in progress
   */
  stopSearch: function() {
   	this._request.abort();
  },

	processResults : function(searchString, stub, listener) {
		var obj = this;
		var searchString = searchString.toLowerCase();
		var comments = [];
		var results = [];
		
		for (var i = 0; i < stub.length; i++){
			if (stub[i].toLowerCase().indexOf(searchString) == 0){
				results.push(stub[i]);
			}
		}
		
		var newResult = new SimpleAutoCompleteResult(searchString, Ci.nsIAutoCompleteResult.RESULT_SUCCESS, 0, "", results, comments);
		listener.onSearchResult(obj, newResult);
	},
    
  QueryInterface: function(aIID) {
    if (!aIID.equals(Ci.nsIAutoCompleteSearch) && !aIID.equals(Ci.nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

// Factory
var SimpleAutoCompleteSearchFactory = {
  singleton: null,
  createInstance: function (aOuter, aIID) {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    if (this.singleton == null)
      this.singleton = new SimpleAutoCompleteSearch();
    return this.singleton.QueryInterface(aIID);
  }
};

// Module
var SimpleAutoCompleteSearchModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType) {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID) {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return SimpleAutoCompleteSearchFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

// Module initialization
function NSGetModule(aCompMgr, aFileSpec) { return SimpleAutoCompleteSearchModule; }

function logMhoShareMsg1(message) {
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
	consoleService.logStringMessage("Mahalo Share: " + message);
}